<?php

namespace SuperRhino\Utils\Container;

use Psr\Container\ContainerInterface;
use SuperRhino\Utils\Cache\DriverCacheAdapterInterface;
use SuperRhino\Utils\Cache\RedisCacheAdapter;
use SuperRhino\Utils\Cache\MemoryCacheAdapterInterface;
use SuperRhino\Utils\Cache\MemoryCacheAdapter;
use SuperRhino\Utils\Counter\CounterInterface;
use SuperRhino\Utils\Counter\RedisCounterAdapter;
use SuperRhino\Utils\KeyStore\KeyStoreValueInterface;
use SuperRhino\Utils\KeyStore\KeyStoreValueAdapter;
use SuperRhino\Utils\KeyStore\KeyStoreCountInterface;
use SuperRhino\Utils\KeyStore\KeyStoreCountAdapter;

class Container implements ContainerInterface {

    /**
     * @var ContainerInterface
     */
    protected $container;

    /**
     * @param ContainerInterface $container
     */
    public function __construct(ContainerInterface $container) {
        $this->container = $container;
    }

    public function get($id) {
        if ($this->container->has($id)) {
            return $this->container->get($id);
        }
        return $this->getInterior($id);
    }

    public function getInterior(string $id) {
        switch ($id) {
            case DriverCacheAdapterInterface::class:
                return new RedisCacheAdapter();
            case MemoryCacheAdapterInterface::class:
                return new MemoryCacheAdapter();
            case KeyStoreValueInterface::class:
                return new KeyStoreValueAdapter();
            case KeyStoreCountInterface::class:
                return new KeyStoreCountAdapter();
            case CounterInterface::class:
                return new RedisCounterAdapter();
            default:
                return null;
        }
    }

    public function has(string $id): bool {
        if ($this->container->has($id)) {
            return true;
        }
        return $this->hasInterior($id);
    }

    public function hasInterior(string $id): bool {
        switch ($id) {
            case DriverCacheAdapterInterface::class:
            case MemoryCacheAdapterInterface::class:
            case KeyStoreValueInterface::class:
            case KeyStoreCountInterface::class:
            case CounterInterface::class:
                return true;
            default:
                return false;
        }
    }
}